"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.designSystemGenerator = void 0;
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const change_case_1 = require("change-case");
const utils_1 = require("../../../utils");
const messages_1 = require("../../messages");
exports.designSystemGenerator = {
    description: (0, messages_1.generatorDescription)('Generate a new design system from 2 existing base systems'),
    prompts(inquirer, prevAns = {}) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            // TODO: Update these to use when() checks
            const { dsPrefix, basePath1, basePrefix1, basePath2, basePrefix2, force } = yield inquirer.prompt([
                {
                    type: 'input',
                    name: 'dsPrefix',
                    message: 'New design system prefix',
                    default: 'alpha',
                },
                // base system 1 is expected to be our DS (foundation-ui, zero etc.)
                {
                    type: 'input',
                    name: 'basePath1',
                    default: (0, utils_1.getFoundationUIPath)(),
                    message: 'Base design system 1 path',
                },
                {
                    type: 'input',
                    name: 'basePrefix1',
                    default: 'foundation',
                    message: 'Base design system 1 prefix',
                },
                // base system 2 is expected to be third-party DS (e.g. MS fast-components)
                {
                    type: 'input',
                    name: 'basePath2',
                    default: (0, utils_1.getMSFastComponentsPath)(),
                    message: 'Base design system 2 path',
                },
                {
                    type: 'input',
                    name: 'basePrefix2',
                    default: 'fast',
                    message: 'Base design system 2 prefix',
                },
                {
                    type: 'confirm',
                    message: 'Overwrite existing files?',
                    name: 'force',
                    default: true,
                },
            ]);
            return {
                dsPrefix,
                basePath1,
                basePrefix1,
                basePath2,
                basePrefix2,
                force,
            };
        });
    },
    actions: (answers) => {
        const { dsPrefix, basePath1, basePath2, basePrefix1, basePrefix2, force } = answers;
        const outputDir = '../../../../../../foundation/foundation-ds';
        const parentBuildFolder = '../../../../build';
        const parentTsConfigFolder = '../../../..';
        const baseSystem1 = (0, utils_1.getDirectoryInfo)(basePath1);
        if (!baseSystem1) {
            console.error(`Couldn't load base system 1 from ${basePath1}`);
            process.exit();
        }
        const baseSystem2 = (0, utils_1.getDirectoryInfo)(basePath2);
        if (!baseSystem2) {
            console.error(`Couldn't load base system 2 from ${basePath2}`);
            process.exit();
        }
        const exclude = [
            '_common',
            '_config',
            'color',
            'data-grid-datasource',
            'design-system-provider',
            'foundation-testing',
            'options-datasource',
            'patterns',
            'popper',
            'router',
            'styles',
            'utilities',
            'utils',
        ];
        const customComponents = [];
        const baseSystem1Exports = baseSystem1.contents.filter((c) => !exclude.includes(c));
        const baseSystem2Exports = baseSystem2.contents
            .filter((c) => !exclude.includes(c))
            .filter((c) => !baseSystem1Exports.includes(c));
        const baseSystemExportsCombined = (0, utils_1.combineArrays)(baseSystem1Exports, baseSystem2Exports);
        const nonComponentExports = ['_common', '_config'];
        const indexExports = [...baseSystemExportsCombined, ...nonComponentExports];
        indexExports.sort((a, b) => a.toLowerCase() - b.toLowerCase());
        const newPackageName = `@genesislcap/${dsPrefix}-design-system`;
        const distFilename = `${dsPrefix}`; // e.g. 'dist/{{distFilename}}.js',
        const packageJsonDescription = '';
        const packageJsonLicence = 'SEE LICENSE IN license.txt';
        const packageJsonRepository = 'https://github.com/genesislcap/foundation-ui.git';
        const baseSystem2Components = baseSystem2Exports.map((name) => ({
            baseName: name,
            componentClass: (0, change_case_1.pascalCase)(name),
            importedComponent: `${basePrefix2}${(0, change_case_1.pascalCase)(name)}`,
            importSource: baseSystem2.name,
            importedStyles: `${(0, change_case_1.camelCase)(name)}Styles`,
            importedTemplate: `${(0, change_case_1.camelCase)(name)}Template`,
            exportedComponent: `${dsPrefix}${(0, change_case_1.pascalCase)(name)}`,
            exportedStyles: `${dsPrefix}${(0, change_case_1.pascalCase)(name)}Styles`,
            exportedTemplate: `${dsPrefix}${(0, change_case_1.pascalCase)(name)}Template`,
            exportedShadowOptions: `${dsPrefix}${(0, change_case_1.pascalCase)(name)}ShadowOptions`,
            defaultConfig: `default${(0, change_case_1.pascalCase)(name)}Config`,
            customComponent: false,
        }));
        const baseSystem1Components = baseSystem1Exports.map((name) => ({
            baseName: name,
            componentClass: (0, change_case_1.pascalCase)(name),
            importedComponent: customComponents.includes(name)
                ? (0, change_case_1.pascalCase)(`${basePrefix1}${(0, change_case_1.pascalCase)(name)}`)
                : `${basePrefix1}${(0, change_case_1.pascalCase)(name)}`,
            importSource: baseSystem1.name,
            exportedComponent: `${dsPrefix}${(0, change_case_1.pascalCase)(name)}`,
            exportedStyles: `${dsPrefix}${(0, change_case_1.pascalCase)(name)}Styles`,
            exportedTemplate: `${dsPrefix}${(0, change_case_1.pascalCase)(name)}Template`,
            exportedShadowOptions: `${dsPrefix}${(0, change_case_1.pascalCase)(name)}ShadowOptions`,
            importedStyles: `${basePrefix1}${(0, change_case_1.pascalCase)(name)}Styles`,
            importedTemplate: `${basePrefix1}${(0, change_case_1.pascalCase)(name)}Template`,
            importedShadowOptions: `${basePrefix1}${(0, change_case_1.pascalCase)(name)}ShadowOptions`,
            defaultConfig: `default${(0, change_case_1.pascalCase)(name)}Config`,
            customComponent: customComponents.includes(name),
        }));
        const allComponents = [...baseSystem2Components, ...baseSystem1Components];
        allComponents.sort((a, b) => (0, utils_1.defaultSort)(a.baseName.toLowerCase(), b.baseName.toLowerCase()));
        const actions = [];
        allComponents.forEach((c) => {
            actions.push({
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/src/${c.baseName}/index.ts`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/component/index.ts.hbs'),
                data: Object.assign({}, c),
                force,
            });
            actions.push({
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/src/${c.baseName}/${c.baseName}.ts`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/component/component.ts.hbs'),
                data: Object.assign({}, c),
                force,
            });
        });
        return [
            () => {
                const source = path_1.default.resolve(__dirname, `../../templates/design-system/_common`);
                const target = path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/src/_common`);
                (0, utils_1.copyFiles)(source, target);
                return `Copied files form ${source} to ${target}`;
            },
            () => {
                const source = path_1.default.resolve(__dirname, `../../templates/design-system/_config`);
                const target = path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/src/_config`);
                (0, utils_1.copyFiles)(source, target);
                return `Copied files form ${source} to ${target}`;
            },
            {
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/src/_config/values/misc.ts`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/config.misc.ts.hbs'),
                data: {
                    prefix: dsPrefix,
                },
                force,
            },
            {
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/src/design-system-provider/design-system-provider.ts`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/design-system-provider/design-system-provider.ts.hbs'),
                data: {
                    prefix: dsPrefix,
                },
                force,
            },
            {
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/README.md`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/README.md.hbs'),
                force,
            },
            {
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/src/design-system-provider/index.ts`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/design-system-provider/index.ts.hbs'),
                force,
            },
            {
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/src/router/index.ts`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/router/index.ts.hbs'),
                data: {
                    prefix: dsPrefix,
                },
                force,
            },
            {
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/src/router/README.md`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/router/README.md.hbs'),
                data: {
                    prefix: dsPrefix,
                },
                force,
            },
            {
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/src/router/router.ts`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/router/router.ts.hbs'),
                force,
            },
            () => {
                const source = path_1.default.resolve(__dirname, `../../templates/design-system/_common`);
                const target = path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/src/_common`);
                (0, utils_1.copyFiles)(source, target);
                return `Copied files form ${source} to ${target}`;
            },
            () => {
                const source = path_1.default.resolve(__dirname, `../../templates/design-system/patterns`);
                const target = path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/src/patterns`);
                (0, utils_1.copyFiles)(source, target);
                return `Copied files form ${source} to ${target}`;
            },
            () => {
                const source = path_1.default.resolve(__dirname, `../../templates/design-system/utils`);
                const target = path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/src/utils`);
                (0, utils_1.copyFiles)(source, target);
                return `Copied files form ${source} to ${target}`;
            },
            {
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/src/base-components.ts`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/base-components.ts.hbs'),
                data: {
                    prefix: dsPrefix,
                    baseComponents: allComponents,
                },
                force,
            },
            {
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/src/index.ts`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/index.ts.hbs'),
                data: {
                    exports: indexExports,
                },
                force,
            },
            {
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/src/provide-design-system.ts`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/provide-design-system.ts.hbs'),
                data: {
                    prefix: dsPrefix,
                },
                force,
            },
            {
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/.npmignore`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/.npmignore.hbs'),
                force,
            },
            {
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/license.txt`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/license.txt.hbs'),
                force,
            },
            {
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/package.json`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/package.json.hbs'),
                data: {
                    fileName: distFilename,
                    packageName: newPackageName,
                    baseDsPackageName: baseSystem1.name,
                    baseDsPackageVersion: `^${baseSystem1.version}`,
                    parentBuildFolder,
                    packageJsonDescription,
                    packageJsonVersion: baseSystem1.version,
                    packageJsonRepository,
                    packageJsonLicence,
                },
                force,
            },
            {
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/rollup.config.js`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/rollup.config.js.hbs'),
                data: {
                    fileName: distFilename,
                    parentBuildFolder,
                },
                force,
            },
            {
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/tsconfig.json`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/tsconfig.json.hbs'),
                data: {
                    parentTsConfigFolder,
                },
                force,
            },
            {
                type: 'add',
                path: path_1.default.resolve(__dirname, `${outputDir}/${dsPrefix}-design-system/tsconfig.json`),
                templateFile: path_1.default.resolve(__dirname, '../../templates/design-system/tsconfig.json.hbs'),
                data: {
                    parentTsConfigFolder,
                },
                force,
            },
            ...actions,
        ];
    },
};
